package com.dd.admin.business.webSocket;

import cn.stylefeng.guns.core.websocket.config.HandlerType;
import cn.stylefeng.guns.core.websocket.handler.GroupMessageHandler;
import cn.stylefeng.guns.core.websocket.handler.OfflineMessageHandler;
import cn.stylefeng.guns.core.websocket.handler.P2PMessageHandler;
import cn.stylefeng.guns.core.websocket.handler.WeiXinOfflineMessageHandler;
import cn.stylefeng.guns.core.websocket.model.LayimUser;
import cn.stylefeng.guns.generator.core.util.StringUtil;
import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.tio.core.ChannelContext;
import org.tio.core.Tio;
import org.tio.http.common.HttpRequest;
import org.tio.http.common.HttpResponse;
import org.tio.utils.lock.SetWithLock;
import org.tio.websocket.common.WsRequest;
import org.tio.websocket.server.handler.IWsMsgHandler;
import org.tio.websocket.starter.TioWebSocketServerBootstrap;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@Component
public class MyWebSocketMsgHandler implements IWsMsgHandler {

    cn.stylefeng.guns.core.websocket.MyWebSocketMsgHandler handler;

    private static Logger log = LoggerFactory.getLogger(cn.stylefeng.guns.core.websocket.MyWebSocketMsgHandler.class);



    @PostConstruct
    public void init() {
        handler = this;
    }

    @Autowired
    public TioWebSocketServerBootstrap bootstrap;

    private Map handlerMap = new HashMap();

    private MyWebSocketMsgHandler() {
        handlerMap.put(HandlerType.P2P_REQ,new P2PMessageHandler());
        handlerMap.put(HandlerType.GROUP_MSG_REQ,new GroupMessageHandler());
        handlerMap.put(HandlerType.OFFLINE_MESSAGE_REQ_MSG,new OfflineMessageHandler());
        handlerMap.put(HandlerType.WEIXIN_OFFLINE_MESSAGE_REQ_MSG,new WeiXinOfflineMessageHandler());
    }


    @Override
    public HttpResponse handshake(HttpRequest request, HttpResponse httpResponse, ChannelContext channelContext) throws Exception {
        String user = request.getParam("user");
        String clientip = request.getClientIp();
        LayimUser layimUser = JSON.parseObject(user,LayimUser.class);
        String id =  layimUser.getId();
        String username = layimUser.getUsername();

        Tio.bindUser(channelContext,id);
        channelContext.setAttribute("name",username);
        channelContext.setAttribute("ip",clientip);

        log.info("user"+user);

        System.out.println(username+"：进入了Tio id："+id+"name："+username+" ip："+clientip);

        SetWithLock<ChannelContext> channelContexts =  Tio.getAllChannelContexts(bootstrap.getServerGroupContext());
        Set<ChannelContext> contextList = channelContexts.getObj();
        System.out.println("当前在线用户:");
        for(ChannelContext context:contextList){
            System.out.println(context.getAttribute("name")+"\t");
        }

       Integer count = channelContexts.size();


        return httpResponse;
    }

    @Override
    public void onAfterHandshaked(HttpRequest httpRequest, HttpResponse httpResponse, ChannelContext channelContext) throws Exception {
//        System.out.println("握手成功进入群组");

    }

    @Override
    public Object onBytes(WsRequest wsRequest, byte[] bytes, ChannelContext channelContext) throws Exception {
        System.out.println("接收到bytes消息");
        return null;
    }

    @Override
    public Object onClose(WsRequest wsRequest, byte[] bytes, ChannelContext channelContext) throws Exception {
        return null;
    }

    @Override
    public Object onText(WsRequest wsRequest, String text, ChannelContext channelContext) throws Exception {
        if(text.equals("心跳内容")) return null;

        System.out.println("接收到文本消息："+text);

        Map map = JSON.parseObject(text,Map.class);

        String handlerType =(String)map.get("hanlderType");

        if(!StringUtil.isEmpty(handlerType)){
            MsgHandlerInterface msgHandler = (MsgHandlerInterface) handlerMap.get(handlerType);
            if(msgHandler!=null){
                msgHandler.handler(map,channelContext);
            }else{
                log.debug("非法请求...");
            }
        }else{
            log.debug("非法请求...");
        }


        System.out.println(map);
        return null;
    }
}
