package com.dd.admin.business.webSocket.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.stylefeng.guns.core.websocket.MsgHandlerInterface;
import cn.stylefeng.guns.core.websocket.config.HandlerType;
import cn.stylefeng.guns.core.websocket.util.TioUtil;
import cn.stylefeng.guns.modular.messgae.model.params.ChatmessageParam;
import cn.stylefeng.guns.modular.messgae.service.ChatmessageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.tio.core.ChannelContext;
import org.tio.core.Tio;
import org.tio.utils.lock.SetWithLock;

import javax.annotation.PostConstruct;
import java.util.Date;
import java.util.Map;
import java.util.Set;


@Component
@Slf4j
public class P2PMessageHandler implements MsgHandlerInterface {

    public static P2PMessageHandler handler;

    @Autowired
    ChatmessageService chatmessageService;

    @PostConstruct
    public void init() {
        handler = this;
        handler.chatmessageService = this.chatmessageService;
    }



    @Override
    public Object handler(Map map, ChannelContext context ){
        Map<String, String> mine = (Map) map.get("mine");
        Map<String, String> to = (Map) map.get("to");

        String fromId = mine.get("id");
        String content = mine.get("content");
        String avatar = mine.get("avatar");
        String userName = mine.get("username");

        String type = to.get("type");
        String toId = to.get("id");
        boolean isMine = false;
        Long timestamp = new Date().getTime();

        ChatmessageParam message = new ChatmessageParam();
        message.setUsername(userName);
        message.setAvatar(avatar);
        message.setContent(content);
        message.setToUid(Long.valueOf(toId));
        message.setType(type);
        message.setTimestamp(timestamp);
        message.setFromid(Long.valueOf(fromId));
//        handler.chatmessageService.add(message);

        Map messageMap = BeanUtil.beanToMap(message);
        messageMap.put("mine",false);
        messageMap.put("id",fromId);


        //t-io支持多点登录,获取的是一个集合,因为此账号可能存在多个连接哦
        SetWithLock<ChannelContext> contexts = Tio.getChannelContextsByUserid(context.getGroupContext(), toId);
        //用户在线
        if(contexts!=null && contexts.size() > 0) {
            Set<ChannelContext> contextList = contexts.getObj();
            //t-io支持多点登录,获取的是一个集合,向集合发送聊天信息
            for (ChannelContext con : contextList) {
                TioUtil.sendMessage(con, HandlerType.P2P_REQ, messageMap);
                message.setUnreadPoint(1);
            }
        }else{
            message.setUnreadPoint(2);
        }
        handler.chatmessageService.add(message);

        return null;
    }}
